@@ -25,6 +25,7 @@
 
 #include <gdk/gdkkeysyms.h>
 #include <games-frame.h>
+#include <games-controls.h>
 
 #include "preferences.h"
 #include "main.h"
@@ -40,19 +41,9 @@ extern GtkWidget *window;
 extern GnibblesProperties *properties;
 extern gint paused;
 
-GtkWidget *control_table[NUMWORMS];
-GtkWidget *control_button[NUMWORMS][4];
 GtkWidget *start_level_label, 
 	*start_level_spin_button;
 
-static gchar *
-keyboard_string (gint ksym)
-{
-	gchar *name;
-	name = gdk_keyval_name (ksym);
-	return name;
-}
-
 static void
 destroy_cb (GtkWidget *widget, gpointer data)
 {
@@ -66,8 +57,6 @@ destroy_cb (GtkWidget *widget, gpointer data)
 static void
 apply_cb (GtkWidget *widget, gint action, gpointer data)
 {
-	update_score_state ();
-
 	gtk_widget_destroy (widget);
 }
 
@@ -115,19 +104,28 @@ random_order_cb (GtkWidget *widget, gpointer data)
 static void
 fake_bonus_cb (GtkWidget *widget, gpointer data)
 {
+	gboolean value;
+
 	if (!pref_dialog)
 		return;
 
-	gnibbles_properties_set_fakes (GPOINTER_TO_INT(data));
+  value = GTK_TOGGLE_BUTTON (widget)->active;
+
+  gnibbles_properties_set_fakes (value);
 }
 
 static void
 sound_cb (GtkWidget *widget, gpointer data)
 {
+	gboolean value;
+
 	if (!pref_dialog)
 		return;
 
-	gnibbles_properties_set_sound (GPOINTER_TO_INT(data));
+	value = GTK_TOGGLE_BUTTON (widget)->active;
+
+	gnibbles_properties_set_sound (value);
+
 }
 
 static void
@@ -143,77 +141,6 @@ num_worms_cb (GtkWidget *widget, gpointer data)
 	gnibbles_properties_set_worms_number (value);
 }
 
-static gint
-worm_up_cb (GtkWidget *widget, GdkEventKey *event, gpointer data)
-{
-	gchar *key_name;
-
-	if (!pref_dialog)
-		return FALSE;
-	
-	key_name = keyboard_string (event->keyval);
-	gtk_entry_set_text (GTK_ENTRY (widget), key_name);
-	gtk_widget_set_sensitive (widget, FALSE);
-	gtk_widget_grab_focus (control_button[GPOINTER_TO_INT(data)][0]);
-	
-	gnibbles_properties_set_worm_up (GPOINTER_TO_INT(data), key_name);
-
-	return TRUE;
-}
-
-static gint
-worm_down_cb (GtkWidget *widget, GdkEventKey *event, gpointer data)
-{
-	gchar *key_name;
-
-	if (!pref_dialog)
-		return FALSE;
-	
-	key_name = keyboard_string (event->keyval);
-	gtk_entry_set_text (GTK_ENTRY (widget), key_name);
-	gtk_widget_set_sensitive (widget, FALSE);
-	gtk_widget_grab_focus (control_button[GPOINTER_TO_INT(data)][1]);
-
-	gnibbles_properties_set_worm_down (GPOINTER_TO_INT(data), key_name);
-
-	return TRUE;
-}
-
-static gint
-worm_left_cb (GtkWidget *widget, GdkEventKey *event, gpointer data)
-{
-	gchar *key_name;
-	if (!pref_dialog)
-		return FALSE;
-	
-	key_name = keyboard_string (event->keyval);
-	gtk_entry_set_text (GTK_ENTRY (widget), key_name);
-	gtk_widget_set_sensitive (widget, FALSE);
-	gtk_widget_grab_focus (control_button[GPOINTER_TO_INT(data)][2]);
-
-	gnibbles_properties_set_worm_left (GPOINTER_TO_INT(data), key_name);
-
-	return TRUE;
-}
-
-static gint
-worm_right_cb (GtkWidget *widget, GdkEventKey *event, gpointer data)
-{
-	gchar *key_name;
-	if (!pref_dialog)
-		return FALSE;
-	
-	key_name = keyboard_string (event->keyval);
-	gtk_entry_set_text (GTK_ENTRY (widget), key_name);
-	gtk_widget_set_sensitive (widget, FALSE);
-	gtk_widget_grab_focus (control_button[GPOINTER_TO_INT(data)][3]);
-
-	gnibbles_properties_set_worm_right (GPOINTER_TO_INT(data), 
-                                            key_name);
-
-	return TRUE;
-}
-
 static void
 set_worm_color_cb (GtkWidget *widget, gpointer data)
 {
@@ -226,18 +153,12 @@ set_worm_color_cb (GtkWidget *widget, gpointer data)
 static void
 set_worm_controls_sensitivity (gint i, gboolean value)
 {
-	const gchar *name;
-	GList *list = NULL;
+	/* FIXME */
 
-	list = gtk_container_get_children (GTK_CONTAINER (control_table[i]));
-	list = g_list_reverse (list);
-
-	for (; list; list = list->next) {
-		name = gtk_widget_get_name (GTK_WIDGET (list->data));
-		if (g_strrstr (name, "UpLabel") || g_strrstr (name, "DownLabel"))
-			gtk_widget_set_sensitive (list->data, !value);
-	}
-	g_list_free (list);
+	/* This is meant to make the up and down controls 
+	 * unavailable if we are in relative mode. However
+	 * The new key selection API doesn't support this
+	 * yet. */
 }
 
 static void
@@ -256,13 +177,6 @@ worm_relative_movement_cb (GtkWidget *widget, gpointer data)
 		(i, GTK_TOGGLE_BUTTON (widget)->active);
 }
 
-static void
-key_change_cb (GtkWidget * widget, GtkWidget * target)
-{
-	gtk_widget_set_sensitive (target, TRUE);
-	gtk_widget_grab_focus (target);
-}
-
 void
 gnibbles_preferences_cb (GtkWidget *widget, gpointer data)
 {
@@ -275,11 +189,10 @@ gnibbles_preferences_cb (GtkWidget *widget, gpointer data)
 	GtkObject *adjustment;
 	GtkWidget *label2;
 	GtkWidget *table, *table2;
-	GtkWidget *entry;
 	GtkWidget *omenu;
-	GtkWidget *entries[NUMWORMS][4];
+	GtkWidget *controls;
 	gchar *buffer;
-	gint i, j;
+	gint i;
 	gint running = 0;
 	
 	if (pref_dialog) {
@@ -295,18 +208,22 @@ gnibbles_preferences_cb (GtkWidget *widget, gpointer data)
 	if (game_running ())
 		running = 1;
 
-	pref_dialog = gtk_dialog_new_with_buttons (_("Gnibbles Preferences"),
+	pref_dialog = gtk_dialog_new_with_buttons (_("Nibbles Preferences"),
                                                    GTK_WINDOW(window), 0,
                                                    GTK_STOCK_CLOSE,
                                                    GTK_RESPONSE_CLOSE, NULL);
+	gtk_dialog_set_has_separator (GTK_DIALOG (pref_dialog), FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (pref_dialog), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (pref_dialog)->vbox), 2);
 
 	notebook = gtk_notebook_new ();
+	gtk_container_set_border_width (GTK_CONTAINER (notebook), 5);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (pref_dialog)->vbox),
                            notebook);
 
-	label = gtk_label_new_with_mnemonic (_("_Game"));
+	label = gtk_label_new (_("Game"));
 	table = gtk_table_new (1, 2, FALSE);
-	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 18);
 	gtk_container_set_border_width (GTK_CONTAINER (table), 12);
 
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
@@ -316,10 +233,9 @@ gnibbles_preferences_cb (GtkWidget *widget, gpointer data)
 	if (running)
 		gtk_widget_set_sensitive (frame, FALSE);
 
-	gtk_table_attach_defaults (GTK_TABLE (table), frame, 0, 1, 0, 1);
+	gtk_table_attach (GTK_TABLE (table), frame, 0, 1, 0, 1, 0, GTK_FILL | GTK_EXPAND, 0, 0);
         
 	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
 	gtk_container_add (GTK_CONTAINER (frame), vbox);
 
 	button = gtk_radio_button_new_with_label (NULL, _("Nibbles newbie"));
@@ -368,10 +284,9 @@ gnibbles_preferences_cb (GtkWidget *widget, gpointer data)
 	gtk_table_attach_defaults (GTK_TABLE (table), frame, 1, 2, 0, 1);
 
         vbox = gtk_vbox_new (FALSE, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
         gtk_container_add (GTK_CONTAINER (frame), vbox);
 
-	button = gtk_check_button_new_with_label (_("Play levels in random order"));
+	button = gtk_check_button_new_with_mnemonic (_("_Play levels in random order"));
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 
 	if (running)
@@ -382,7 +297,7 @@ gnibbles_preferences_cb (GtkWidget *widget, gpointer data)
 	g_signal_connect (GTK_OBJECT (button), "toggled", GTK_SIGNAL_FUNC
                           (random_order_cb), NULL);
 
-	button = gtk_check_button_new_with_label (_("Enable fake bonuses"));
+	button = gtk_check_button_new_with_mnemonic (_("_Enable fake bonuses"));
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 
 	if (running)
@@ -393,7 +308,7 @@ gnibbles_preferences_cb (GtkWidget *widget, gpointer data)
 	g_signal_connect (GTK_OBJECT (button), "toggled", GTK_SIGNAL_FUNC
                           (fake_bonus_cb), NULL);
         
-	button = gtk_check_button_new_with_label (_("Enable sounds"));
+	button = gtk_check_button_new_with_mnemonic (_("E_nable sounds"));
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 	if (properties->sound)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
@@ -404,9 +319,10 @@ gnibbles_preferences_cb (GtkWidget *widget, gpointer data)
 	table2 = gtk_table_new (2, 2, FALSE);
 	gtk_box_pack_start (GTK_BOX (vbox), table2, FALSE, FALSE, 0);
 	gtk_table_set_row_spacings (GTK_TABLE (table2), 6);
+	gtk_table_set_col_spacings (GTK_TABLE (table2), 12);
 	gtk_container_set_border_width (GTK_CONTAINER (table2), 0);
 
-	label2 = gtk_label_new (_("Starting level:"));
+	label2 = gtk_label_new_with_mnemonic (_("_Starting level:"));
 	start_level_label = label2;
 	gtk_widget_set_name (label2, "StartLevelLabel");
 	gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
@@ -415,7 +331,7 @@ gnibbles_preferences_cb (GtkWidget *widget, gpointer data)
 		gtk_widget_set_sensitive (GTK_WIDGET (label2), FALSE);
 	if (running)
 		gtk_widget_set_sensitive (GTK_WIDGET (label2), FALSE);
-	gtk_table_attach_defaults (GTK_TABLE (table2), label2, 0, 1, 0, 1);
+	gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
 
 	adjustment = gtk_adjustment_new ((gfloat) properties->startlevel, 1.0,
 					 MAXLEVEL, 1.0, 5.0, 0.0);
@@ -424,6 +340,7 @@ gnibbles_preferences_cb (GtkWidget *widget, gpointer data)
 	start_level_spin_button = levelspinner;
 	gtk_widget_set_name (levelspinner, "StartLevelSpinButton");
 	gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (levelspinner), FALSE);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label2), levelspinner);
 
 	if (properties->random)
 		gtk_widget_set_sensitive (GTK_WIDGET (levelspinner), FALSE);
@@ -433,10 +350,10 @@ gnibbles_preferences_cb (GtkWidget *widget, gpointer data)
 	g_signal_connect (GTK_OBJECT (adjustment), "value_changed",
 			  GTK_SIGNAL_FUNC (start_level_cb), levelspinner);
 
-	label2 = gtk_label_new (_("Number of players:"));
+	label2 = gtk_label_new_with_mnemonic (_("N_umber of players:"));
 	gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
 
-	gtk_table_attach_defaults (GTK_TABLE (table2), label2, 0, 1, 1, 2);
+	gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
 	if (running)
 		gtk_widget_set_sensitive (label2, FALSE);
 
@@ -445,6 +362,7 @@ gnibbles_preferences_cb (GtkWidget *widget, gpointer data)
 
 	button = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 0, 0);
 	gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (button), FALSE);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label2), button);
 
 	gtk_table_attach_defaults (GTK_TABLE (table2), button, 1, 2, 1, 2);
 	if (running)
@@ -453,120 +371,59 @@ gnibbles_preferences_cb (GtkWidget *widget, gpointer data)
                           GTK_SIGNAL_FUNC (num_worms_cb), button);
         
 	for (i = 0; i < NUMWORMS; i++) {
-		buffer = g_strdup_printf ("%s _%d", _("Worm"), i + 1);
-		label = gtk_label_new_with_mnemonic (buffer);
+		gchar * up_key;
+		gchar * down_key;
+		gchar * left_key;
+		gchar * right_key;
+
+		buffer = g_strdup_printf ("%s %d", _("Worm"), i + 1);
+		label = gtk_label_new (buffer);
                 g_free (buffer);
 
-                vbox = gtk_vbox_new (FALSE, 6);
+                vbox = gtk_vbox_new (FALSE, 18);
                 gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
 
 		gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);
 
-                frame = games_frame_new (_("Keyboard controls"));
-                gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
+                frame = games_frame_new (_("Keyboard Controls"));
 
-		table = gtk_table_new (4, 5, FALSE);
-		control_table[i] = table;
-                gtk_container_add (GTK_CONTAINER (frame), table);
-        
-		gtk_container_set_border_width (GTK_CONTAINER (table), 0);
-		gtk_table_set_col_spacings (GTK_TABLE (table), 0);
-		gtk_table_set_row_spacings (GTK_TABLE (table), 0);
-		
-		control_button[i][0] = gtk_button_new_with_label (_("Up"));
-		gtk_widget_set_name (control_button[i][0],
-				     "WormControlUpLabel");
-		gtk_table_attach (GTK_TABLE (table), control_button[i][0],
-				  2, 3, 0, 1, 0, 0 , 3, 3);
+		controls = games_controls_list_new ();
+	
+		left_key = g_strdup_printf (KEY_WORM_LEFT, i);
+		right_key = g_strdup_printf (KEY_WORM_RIGHT, i);
+		up_key = g_strdup_printf (KEY_WORM_UP, i);
+		down_key = g_strdup_printf (KEY_WORM_DOWN, i);
 
-		entry = gtk_entry_new ();
-		gtk_widget_set_name (entry, "WormControlUpEntry");
-		gtk_entry_set_text (GTK_ENTRY (entry), properties->wormprops[i]->up);
-		gtk_editable_set_editable (GTK_EDITABLE (entry), FALSE);
-                gtk_entry_set_width_chars (GTK_ENTRY (entry), KB_TEXT_NCHARS);
-		gtk_table_attach (GTK_TABLE (table), entry, 2, 3, 1, 2, 0, 0, 3, 3);
-		g_signal_connect (GTK_OBJECT (entry), "key_press_event",
-                                  GTK_SIGNAL_FUNC (worm_up_cb), GINT_TO_POINTER(i));
-		g_signal_connect (GTK_OBJECT (control_button[i][0]),
-				  "clicked",
-				  GTK_SIGNAL_FUNC(key_change_cb),
-				  entry);
-		entries[i][0] = entry;
-		
-		control_button[i][1] = gtk_button_new_with_label (_("Down"));
-		gtk_widget_set_name (control_button[i][1],
-				     "WormControlDownLabel");
-		gtk_table_attach (GTK_TABLE (table), control_button[i][1],
-				  2, 3, 4, 5, 0, 0, 3, 3);
-
-		entry = gtk_entry_new ();
-		gtk_widget_set_name (entry, "WormControlDownEntry");
-		gtk_entry_set_text (GTK_ENTRY (entry), properties->wormprops[i]->down);
-		gtk_editable_set_editable (GTK_EDITABLE (entry), FALSE);
-                gtk_entry_set_width_chars (GTK_ENTRY (entry), KB_TEXT_NCHARS);
-		gtk_table_attach (GTK_TABLE (table), entry, 2, 3, 3, 4, 0, 0, 3, 3);
-		g_signal_connect (GTK_OBJECT (entry), "key_press_event",
-                                  GTK_SIGNAL_FUNC (worm_down_cb), GINT_TO_POINTER(i));
-		g_signal_connect (GTK_OBJECT (control_button[i][1]),
-				  "clicked",
-				  GTK_SIGNAL_FUNC(key_change_cb),
-				  entry);
-		entries[i][1] = entry;
-
-		control_button[i][2] = gtk_button_new_with_label (_("Left"));
-		gtk_table_attach (GTK_TABLE (table),
-				  control_button[i][2],
-				  0, 1, 2, 3, 0, 0, 3, 3);
-
-		entry = gtk_entry_new ();
-		gtk_entry_set_text (GTK_ENTRY (entry), properties->wormprops[i]->left);
-		gtk_editable_set_editable (GTK_EDITABLE (entry), FALSE);
-                gtk_entry_set_width_chars (GTK_ENTRY (entry), KB_TEXT_NCHARS);
-		gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 2, 3, 0, 0, 3, 3);
-
-		g_signal_connect (GTK_OBJECT (entry), "key_press_event",
-                                  GTK_SIGNAL_FUNC (worm_left_cb), GINT_TO_POINTER(i));
-		g_signal_connect (GTK_OBJECT (control_button[i][2]),
-				  "clicked",
-				  GTK_SIGNAL_FUNC(key_change_cb),
-				  entry);
-		entries[i][2] = entry;
-
-		control_button[i][3] = gtk_button_new_with_label (_("Right"));
-		gtk_table_attach (GTK_TABLE (table), control_button[i][3],
-				  4, 5, 2, 3, 0, 0, 3, 3);
-
-		entry = gtk_entry_new ();
-		gtk_entry_set_text (GTK_ENTRY (entry), properties->wormprops[i]->right);
-		gtk_editable_set_editable (GTK_EDITABLE (entry), FALSE);
-                gtk_entry_set_width_chars (GTK_ENTRY (entry), KB_TEXT_NCHARS);
-		gtk_table_attach (GTK_TABLE (table), entry, 3, 4, 2, 3, 0, 0, 3, 3);
-		g_signal_connect (GTK_OBJECT (entry), "key_press_event",
-                                  GTK_SIGNAL_FUNC (worm_right_cb), GINT_TO_POINTER(i));
-		g_signal_connect (GTK_OBJECT (control_button[i][3]),
-				  "clicked",
-				  GTK_SIGNAL_FUNC(key_change_cb),
-				  entry);
-		entries[i][3] = entry;
+		games_controls_list_add_controls (GAMES_CONTROLS_LIST (controls), left_key, right_key, up_key, down_key, NULL);
+		gtk_container_add (GTK_CONTAINER (frame), controls);
                 
+		g_free (left_key);
+		g_free (right_key);
+		g_free (up_key);
+		g_free (down_key);
+
+                gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
+
                 frame = games_frame_new (_("Options"));
                 gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
 
                 vbox2 = gtk_vbox_new (FALSE, 6);
                 gtk_container_add (GTK_CONTAINER (frame), vbox2);
 
-		button = gtk_check_button_new_with_label
-			(_("Use relative movement"));
+		button = gtk_check_button_new_with_mnemonic
+			(_("_Use relative movement"));
                 gtk_box_pack_start (GTK_BOX (vbox2), button, FALSE, FALSE, 0);
 
                 table2 = gtk_table_new (1, 2, FALSE);
+		gtk_table_set_col_spacings (GTK_TABLE (table2), 12);
                 gtk_box_pack_start (GTK_BOX (vbox2), table2, FALSE, FALSE, 0);
 
-		label2 = gtk_label_new (_("Worm color:"));
+		label2 = gtk_label_new_with_mnemonic (_("_Worm color:"));
 		gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
-                gtk_table_attach_defaults (GTK_TABLE (table2), label2, 0, 1, 0, 1);
+                gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 0, 1, 0, 0, 0, 0);
 
 		omenu = gtk_combo_box_new_text ();
+		gtk_label_set_mnemonic_widget (GTK_LABEL (label2), omenu);
 		gtk_combo_box_append_text (GTK_COMBO_BOX (omenu), _("Red"));
 		gtk_combo_box_append_text (GTK_COMBO_BOX (omenu), _("Green"));
 		gtk_combo_box_append_text (GTK_COMBO_BOX (omenu), _("Blue"));
@@ -598,8 +455,4 @@ gnibbles_preferences_cb (GtkWidget *widget, gpointer data)
 	
 	gtk_widget_show_all (pref_dialog);
 
-	for (i = 0; i<NUMWORMS; i++)
-		for (j = 0; j<4; j++)
-			gtk_widget_set_sensitive (entries[i][j], FALSE);
-			
 }
