@@ -69,7 +69,7 @@ GamesScoresBackend *games_scores_backend_new (GamesScoreStyle style,
 					      gchar *basename,
 					      gchar *name);
 GList *games_scores_backend_get_scores (GamesScoresBackend *self);
-void games_scores_backend_set_scores (GamesScoresBackend *self,
+gboolean games_scores_backend_set_scores (GamesScoresBackend *self,
 				      GList *list);
 void games_scores_backend_discard_scores (GamesScoresBackend *self);
 
