@@ -210,10 +210,10 @@ static void parse_start_el (GMarkupParseContext *context,
       if (g_ascii_strncasecmp (*attribute_names, a->name, a->length) == 0) {
 	switch (a->type) {
 	case ATTR_INT:
-	  *(gint *)(a->value) = (gint)(atof (*attribute_values));
+	  *(gint *)(a->value) = (gint)(g_ascii_strtod (*attribute_values, NULL));
 	  break;	  
 	case ATTR_HALFINT:
-	  *(gint *)(a->value) = (gint)(2*atof (*attribute_values));
+	  *(gint *)(a->value) = (gint)(2*g_ascii_strtod (*attribute_values, NULL));
 	  break;
 	case ATTR_STRING:
 	  *(const gchar **)(a->value) = *attribute_values;
